<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 2/2/20
 * Time: 9:02 AM
 */

namespace IMATHUZH\Qfq\Core\Typo3;

use IMATHUZH\Qfq\Core\Helper\Support;


/**
 * Class T3Handler
 * @package IMATHUZH\Qfq\Core\Typo3
 */
class T3Handler {

    /**
     * @var
     */
    private static $objectManager = null;

    /**
     * Call logoff of current FE User
     */
    public static function feLogOff() {

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        $GLOBALS['TSFE']->fe_user->logoff();

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();
    }


    /**
     * Based on https://docs.typo3.org/typo3cms/extensions/saltedpasswords/8.7/DevelopersGuide/Index.html
     * Convert a cleartext password to a hash. Respects if 'salted passwords' are enabled.
     *
     * @param string $newPassword
     * @return string
     */
    public static function getHash($newPassword) {

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        $saltedPassword = md5($newPassword);  // Use md5 as fallback
        self::t3AutoloadIfNotRunning();
        if (\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility::isUsageEnabled('FE')) {
            $objSalt = \TYPO3\CMS\Saltedpasswords\Salt\SaltFactory::getSaltingInstance(NULL);
            if (is_object($objSalt)) {
                $saltedPassword = $objSalt->getHashedPassword($newPassword);
            }
        }

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();

        return $saltedPassword;
    }

    /**
     * Based on https://docs.typo3.org/typo3cms/extensions/saltedpasswords/8.7/DevelopersGuide/Index.html
     * Check if the salted password corresponds to the password.
     *
     * @param string $saltedPassword
     * @param string $password
     * @return bool
     */
    public static function checkPassword($saltedPassword, $password) {

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        self::t3AutoloadIfNotRunning();
        $success = FALSE;
        if (\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility::isUsageEnabled('FE')) {
            $objSalt2 = \TYPO3\CMS\Saltedpasswords\Salt\SaltFactory::getSaltingInstance($saltedPassword);
            if (is_object($objSalt2)) {
                $success = $objSalt2->checkPassword($password, $saltedPassword);
            }
        }

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();

        return $success;
    }

    /**
     * Load Typo3 autoloader if Typo3 is not instantiated
     */
    public static function t3AutoloadIfNotRunning() {

        if (!class_exists('\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility')) {
            //TODO: Get absolute 'path' via QFQ config
            require __DIR__ . '/../../../../../../typo3_src/vendor/autoload.php';

            // run typo3 bootstrap if not yet happened. Necessary if run in unittest.
            if (!defined('TYPO3_MODE')) {
                \TYPO3\CMS\Core\Core\Bootstrap::getInstance()
                    ->setRequestType(TYPO3_REQUESTTYPE_AJAX)
                    ->baseSetup(0);
//                Alternate error fix if you don't want to run Typo3 bootstrap:
//                error_reporting(E_ALL & ~(E_STRICT | E_NOTICE | E_DEPRECATED));
                define('TYPO3_MODE', 'FE');
            }
        }
    }

    /**
     * Update a single key/value pair in `typo3conf/LocalConfiguration.php` QFQ config.
     *
     * @param $key
     * @param $value
     */
    public static function updateT3QfqConfig($key, $value) {

        if (defined('PHPUNIT_QFQ')) {
            // There is no  typo3conf/LocalConfiguration.php in phpunit.
            return;
        }

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        self::t3AutoloadIfNotRunning();

        $configurationManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Configuration\\ConfigurationManager');

        // Same as $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['qfq']
        $configT3 = $configurationManager->getLocalConfiguration();
        $configQfq = unserialize($configT3['EXT']['extConf'][EXT_KEY]);

        // Set new value
        $configQfq[$key] = $value;

        // Prepare
        $configT3['EXT']['extConf'][EXT_KEY] = serialize($configQfq);

        // Write new config to typo3conf/LocalConfiguration.php
        $configurationManager->writeLocalConfiguration($configT3);

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();
    }
}